#!/bin/bash

set -e

cd "${APP_INSTALL_PATH}"

# check that all environment variables are set
requiredVars=(
  OBFSVPN_STATE
  OBFSVPN_LOCATION
  OBFS4_IP
  OBFS4_PORT
  OBFS4_HOST
  OPENVPN_HOST
  OPENVPN_PORT
  OBFS4_DATA_DIR
  OBFS4_KEY_FILE
)

for name in "${requiredVars[@]}"; do
    if [[ -z "${!name}" ]]; then
        echo "Required environment variable $name not set!"
        QUIT=true
    fi
done
if [[ ${QUIT} ]]; then
    echo -e "\nStarting obfsvpn failed.\n"
    exit 1;
fi

echo ">>>> LIVE_TEST: ${LIVE_TEST}"
echo ">>>> starting obfsvpn server - pointing to gateway ${OPENVPN_HOST}:${OPENVPN_PORT}"

if [[ "$HOP_PT" == "1" ]]; then
    if [[ "$KCP" == "1" ]]; then
      # start the obfsvpn server in udp hopping mode + kcp
      ./obfsvpn-server --hop \
          --kcp \
          --addr "${OBFS4_HOST}" \
          --remote "${OPENVPN_HOST}:${OPENVPN_PORT}" \
          --state "$OBFS4_DATA_DIR" \
          --config "${OBFS4_KEY_FILE:-$OBFS4_DATA_DIR/obfs4.json}" \
          --persist="${PERSIST_BRIDGE_STATE:-false}" \
          --location="${OBFSVPN_LOCATION}" \
          --hostname="$(hostname)" \
          --menshen-agent-enabled="true" \
          --menshen-url="http://menshen:${MENSHEN_PORT}" \
          --menshen-agent-key="${MENSHEN_SHARED_KEY}" \
          -v
    elif [[ "$QUIC" == "1" ]]; then
      # start the obfsvpn server in hopping + quic mode
      ./obfsvpn-server --hop \
          --quic \
          --quic-tls-cert-file "${QUIC_TLS_CERT}" \
          --quic-tls-key-file "${QUIC_TLS_KEY}" \
          --addr "${OBFS4_HOST}" \
          --remote "${OPENVPN_HOST}:${OPENVPN_PORT}" \
          --state "$OBFS4_DATA_DIR" \
          --config "${OBFS4_KEY_FILE:-$OBFS4_DATA_DIR/obfs4.json}" \
          --persist="${PERSIST_BRIDGE_STATE:-false}" \
          --location="${OBFSVPN_LOCATION}" \
          --hostname="$(hostname)" \
          --menshen-agent-enabled="true" \
          --menshen-url="http://menshen:${MENSHEN_PORT}" \
          --menshen-agent-key="${MENSHEN_SHARED_KEY}" \
          -v
    fi
    # start the obfsvpn server in udp hopping mode
    ./obfsvpn-server --hop \
        --addr "${OBFS4_HOST}" \
        --remote "${OPENVPN_HOST}:${OPENVPN_PORT}" \
        --state "$OBFS4_DATA_DIR" \
        --config "${OBFS4_KEY_FILE:-$OBFS4_DATA_DIR/obfs4.json}" \
        --persist="${PERSIST_BRIDGE_STATE:-false}" \
        --location="${OBFSVPN_LOCATION}" \
        --hostname="$(hostname)" \
        --menshen-agent-enabled="true" \
        --menshen-url="http://menshen:${MENSHEN_PORT}" \
        --menshen-agent-key="${MENSHEN_SHARED_KEY}" \
        -v
elif [[ "$KCP" == "1" ]]; then
    # start the obfsvpn server in obfs4 kcp mode
    ./obfsvpn-server \
        --kcp \
        --udp \
        --addr "${OBFS4_HOST}" \
        --port "${OBFS4_PORT}" \
        --remote "${OPENVPN_HOST}:${OPENVPN_PORT}" \
        --state "$OBFS4_DATA_DIR" \
        --config "${OBFS4_KEY_FILE:-$OBFS4_DATA_DIR/obfs4.json}" \
        --persist="${PERSIST_BRIDGE_STATE:-false}" \
        --location="${OBFSVPN_LOCATION}" \
        --hostname="$(hostname)" \
        --menshen-agent-enabled="true" \
        --menshen-url="http://menshen:${MENSHEN_PORT}" \
        --menshen-agent-key="${MENSHEN_SHARED_KEY}" \
        -v

elif [[ "$QUIC" == "1" ]]; then
    # start the obfsvpn server in obfs4 quic mode
    ./obfsvpn-server \
        --quic \
        --quic-tls-cert-file "${QUIC_TLS_CERT}" \
        --quic-tls-key-file "${QUIC_TLS_KEY}" \
        --udp \
        --addr "${OBFS4_HOST}" \
        --port "${OBFS4_PORT}" \
        --remote "${OPENVPN_HOST}:${OPENVPN_PORT}" \
        --state "$OBFS4_DATA_DIR" \
        --config "${OBFS4_KEY_FILE:-$OBFS4_DATA_DIR/obfs4.json}" \
        --persist="${PERSIST_BRIDGE_STATE:-false}" \
        --location="${OBFSVPN_LOCATION}" \
        --hostname="$(hostname)" \
        --menshen-agent-enabled="true" \
        --menshen-url="http://menshen:${MENSHEN_PORT}" \
        --menshen-agent-key="${MENSHEN_SHARED_KEY}" \
        -v

elif [[ "$TCP" == "1" ]]; then
    # start the obfsvpn server in obfs4 mode w/ tcp as the protocol the server accepts for proxying traffic.
    ./obfsvpn-server \
        --addr "${OBFS4_HOST}" \
        --port "${OBFS4_PORT}" \
        --remote "${OPENVPN_HOST}:${OPENVPN_PORT}" \
        --state "$OBFS4_DATA_DIR" \
        --config "${OBFS4_KEY_FILE:-$OBFS4_DATA_DIR/obfs4.json}" \
        --persist="${PERSIST_BRIDGE_STATE:-false}" \
        --location="${OBFSVPN_LOCATION}" \
        --hostname="$(hostname)" \
        --menshen-agent-enabled="true" \
        --menshen-url="http://menshen:${MENSHEN_PORT}" \
        --menshen-agent-key="${MENSHEN_SHARED_KEY}" \
        -v
else
     # start the obfsvpn server in regular obfs4 mode
    ./obfsvpn-server \
        --udp \
        --addr "${OBFS4_HOST}" \
        --port "${OBFS4_PORT}" \
        --remote "${OPENVPN_HOST}:${OPENVPN_PORT}" \
        --state "$OBFS4_DATA_DIR" \
        --config "${OBFS4_KEY_FILE:-$OBFS4_DATA_DIR/obfs4.json}" \
        --persist="${PERSIST_BRIDGE_STATE:-false}" \
        --location="${OBFSVPN_LOCATION}" \
        --hostname="$(hostname)" \
        --menshen-agent-enabled="true" \
        --menshen-url="http://menshen:${MENSHEN_PORT}" \
        --menshen-agent-key="${MENSHEN_SHARED_KEY}" \
        -v
fi
